#include <math.h>
#include <stdio.h>
#include <windows.h>
#include "newton.h"
#include "fmod.hpp"

void forceCallback(const NewtonBody* body)
{
	dFloat mass;
	dFloat Ixx;
	dFloat Iyy;
	dFloat Izz;
	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
	dFloat force[3] = {0.0f, 0.0f, -mass*9.81f};
	NewtonBodySetForce(body, force); 
}

FMOD::System *fmodSystem;
FMOD::Sound *s1;

int collisionCallback(const NewtonMaterial* material, const NewtonContact* contact)
{
	float speed = NewtonMaterialGetContactNormalSpeed(material, contact);
	if(speed > 0.0f) // We're coming from the OUTside, not from the INside
	{
		printf("Hit with speed %f\n", speed);
		FMOD::Channel *c1;
		fmodSystem->playSound(FMOD_CHANNEL_FREE, s1, true, &c1);
		c1->setVolume(min(1.0f, max(0.0f, 0.3f*sqrt(speed))));
		c1->setPaused(false);
	}
	return 1;
}

int main(int argc, char* argv[])
{
	FMOD::System_Create(&fmodSystem);
	fmodSystem->init(100, FMOD_INIT_NORMAL, 0);

	fmodSystem->createSound("D:/Dokumente und Einstellungen/Jrn Loviscach/Desktop/FMODdemo/dong.wav", FMOD_CREATESAMPLE, NULL, &s1);

	NewtonWorld* world = NewtonCreate (NULL, NULL);

	NewtonCollision* collision = NewtonCreateSphere (world, 1.0f, 1.0f, 1.0f, NULL); 
	NewtonBody* sphere = NewtonCreateBody (world, collision);
	NewtonReleaseCollision (world, collision);
	NewtonBodySetMassMatrix (sphere, 1.0f, 1.0f, 1.0f, 1.0f);
	dFloat sphereMatrix[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,6,1};
	NewtonBodySetMatrix(sphere, sphereMatrix);
	NewtonBodySetLinearDamping(sphere, 0.00f);

	collision = NewtonCreateBox (world, 10.0f, 10.0f, 1.0f, NULL);
	NewtonBody* block = NewtonCreateBody (world, collision);
	NewtonReleaseCollision (world, collision);
	NewtonBodySetMassMatrix (block, 1000.0f, 1000.0f, 1000.0f, 1000.0f);
	dFloat blockMatrix[16] = {1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1};
	NewtonBodySetMatrix(block, blockMatrix);
	NewtonBodySetLinearDamping(block, 0.00f);

	NewtonBodySetForceAndTorqueCallback(sphere, forceCallback); 

	NewtonMaterialSetCollisionCallback(world, 0, 0, NULL, NULL, collisionCallback, NULL);

	dFloat v[3] = {0,0,-1};
	NewtonBodySetVelocity(sphere, v);

	for(int i = 0; i < 150; i++)
	{
		NewtonUpdate(world, 0.1f);
		Sleep(100);
		NewtonBodyGetMatrix(sphere, sphereMatrix);
		NewtonBodyGetMatrix(block, blockMatrix);
		printf("%f %f %f   %f %f %f\n", sphereMatrix[12], sphereMatrix[13], sphereMatrix[14], blockMatrix[12], blockMatrix[13], blockMatrix[14]);
	}

	NewtonDestroyBody(world, sphere);
	NewtonDestroyBody(world, block);
	NewtonDestroy(world);

	s1->release();
	fmodSystem->release();

	int dummy = getchar();

	return 0;
}

